<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'VP_Woo_Pont_Import_Database', false ) ) :

	class VP_Woo_Pont_Import_Database {
		private static $pont_types = array('foxpost', 'postapont', 'packeta', 'sprinter', 'expressone', 'gls');

		public static function init() {

			//Setup daily scheduled action to refresh databases
			foreach (self::$pont_types as $pont_type) {
				add_action( 'vp_woo_pont_update_'.$pont_type.'_list', array( __CLASS__, 'get_'.$pont_type.'_json' ) );
			}

			//Ajax function to trigger import manually
			add_action( 'wp_ajax_vp_woo_pont_import_json_manually', array( __CLASS__, 'import_manually' ) );

		}

		public static function schedule_actions() {
			foreach (self::$pont_types as $pont_type) {

				//Download postapont list on activation
				WC()->queue()->cancel_all( 'vp_woo_pont_update_'.$pont_type.'_list' );
				WC()->queue()->add( 'vp_woo_pont_update_'.$pont_type.'_list', array(), 'vp_woo_pont' );

				//Setup scheduled actions
				WC()->queue()->schedule_recurring( time(), DAY_IN_SECONDS, 'vp_woo_pont_update_'.$pont_type.'_list', array(), 'vp_woo_pont' );

			}
		}

		public static function get_postapont_json() {
			$request = wp_remote_get('https://www.posta.hu/szolgaltatasok/posta-srv-postoffice/rest/postoffice/listPPMarkers?callback');

			//Check for errors
			if( is_wp_error( $request ) ) {
				VP_Woo_Pont()->log_error_messages($request, 'postapont-import-points');
				return false;
			}

			//Get body
			$body = wp_remote_retrieve_body( $request );

			//Remove first and last character, because it was a jsonp request
			$body = substr($body, 1, -1);

			//Try to convert into json
			$json = json_decode( $body );

			//Check if json exists
			if($json === null) {
				return false;
			}

			//Create a new json
			$results = array();
			$valid_groups = array(10,20,30,50,70); //60 and 80 category is available, but nto documented, so lets skip those for now

			//Simplify json, so its smaller to store, faster to load
			foreach ($json as $postapont) {
				$group = intval(substr($postapont->group, 0, 2));
				if(in_array($group, $valid_groups)) {
					$result = array(
						'id' => $postapont->id,
						'group' => $group,
						'lat' => number_format($postapont->lat, 5, '.', ''),
						'lon' => number_format($postapont->lon, 5, '.', ''),
						'name' => $postapont->name,
						'zip' => $postapont->kzip,
						'addr' => $postapont->address,
						'city' => $postapont->county,
						'comment' => $postapont->phone
					);
					$results[] = $result;
				}
			}

			//Save stuff
			self::save_json_file('postapont', $results);
			return true;

		}

		public static function get_foxpost_json() {
			$request = wp_remote_get('https://cdn.foxpost.hu/foxpost_terminals_extended_v3.json');

			//Check for errors
			if( is_wp_error( $request ) ) {
				VP_Woo_Pont()->log_error_messages($request, 'foxpost-import-points');
				return false;
			}

			//Get body
			$body = wp_remote_retrieve_body( $request );

			//Try to convert into json
			$json = json_decode( $body );

			//Check if json exists
			if($json === null) {
				return false;
			}

			//Create a new json
			$results = array();

			//Simplify json, so its smaller to store, faster to load
			foreach ($json as $foxpost) {
				$result = array(
					'id' => $foxpost->place_id,
					'lat' => number_format($foxpost->geolat, 5, '.', ''),
					'lon' => number_format($foxpost->geolng, 5, '.', ''),
					'name' => $foxpost->name,
					'zip' => $foxpost->zip,
					'addr' => $foxpost->street,
					'city' => $foxpost->city,
					'comment' => $foxpost->findme
				);
				$results[] = $result;
			}

			//Save stuff
			self::save_json_file('foxpost', $results);
			return true;

		}

		public static function get_packeta_json($api_key = false) {

			//Check API key
			if(!$api_key) {
				$api_key = VP_Woo_Pont_Helpers::get_option('packeta_api_key');
			}

			//If theres no api key, return error
			if(!$api_key) {
				return false;
			}

			//Make request for json file
			$download_folder = VP_Woo_Pont_Helpers::get_download_folder('packeta');
			$filename = $download_folder['dir'].'packeta-source-file.json';
			$request = wp_remote_get('https://www.zasilkovna.cz/api/v4/'.$api_key.'/branch.json?lang=hu', array(
				'timeout' => 100,
				'stream' => true,
				'filename' => $filename
			));

			//Check for errors
			if( is_wp_error( $request ) ) {
				VP_Woo_Pont()->log_error_messages($request, 'packeta-import-points');
				return false;
			}

			//Get body
			$body = file_get_contents($filename);

			//Try to convert into json
			$json = json_decode( $body );

			//Check if json exists
			if($json === null) {
				return false;
			}

			//Create a new json
			$results = array();

			//Simplify json, so its smaller to store, faster to load
			foreach ($json->data as $place) {

				//For now, only hungarian points
				if($place->country == 'hu' && $place->status->statusId == '1') {
					$result = array(
						'id' => $place->id,
						'lat' => number_format($place->latitude, 5, '.', ''),
						'lon' => number_format($place->longitude, 5, '.', ''),
						'name' => $place->place,
						'zip' => $place->zip,
						'addr' => $place->street,
						'city' => $place->city
					);

					if(isset($place->directions) && $place->directions != new stdClass()) {
						$result['comment'] = wp_strip_all_tags($place->directions);
					}

					$results[] = $result;
				}

			}

			//Save stuff
			self::save_json_file('packeta', $results);

			//Delete source file
			unlink($filename);

			return true;

		}

		public static function get_sprinter_json() {

			//Make request for json file
			$request = wp_remote_get('https://partner.pickpackpont.hu/stores/ShopList.json');

			//Check for errors
			if( is_wp_error( $request ) ) {
				VP_Woo_Pont()->log_error_messages($request, 'sprinter-import-points');
				return false;
			}

			//Get body
			$body = wp_remote_retrieve_body( $request );

			//Remove BOM, so it sa valid json
			$bom = pack('H*','EFBBBF');
	    $body = preg_replace("/^$bom/", '', $body);

			//Try to convert into json
			$json = json_decode( $body, true );

			//Check if json exists
			if($json === null) {
				return false;
			}

			//Create a new json
			$results = array();

			//Simplify json, so its smaller to store, faster to load
			foreach ($json as $place) {

				//For now, only hungarian points
				$result = array(
					'id' => $place['shopCode'],
					'lat' => number_format($place['lat'], 5, '.', ''),
					'lon' => number_format($place['lng'], 5, '.', ''),
					'name' => $place['shopName'],
					'zip' => $place['zipCode'],
					'addr' => $place['address'],
					'city' => $place['city']
				);

				if($place['description']) {
					$result['comment'] = $place['description'];
				}

				$results[] = $result;
			}

			//Save stuff
			self::save_json_file('sprinter', $results);
			return true;

		}

		public static function get_expressone_json() {

			//Make request for json file
			$request = wp_remote_post('https://webcas.expressone.hu/eoneshops/eoneshop.php?method=getTofShopPointsAll&lang=hu');

			//Check for errors
			if( is_wp_error( $request ) ) {
				VP_Woo_Pont()->log_error_messages($request, 'expressone-import-points');
				return false;
			}

			//Get body
			$body = wp_remote_retrieve_body( $request );

			//Try to convert into json
			$json = json_decode( $body, true );

			//Check if json exists
			if($json === null) {
				return false;
			}

			//Create a new json
			$results = array();

			//Simplify json, so its smaller to store, faster to load
			foreach ($json as $place) {

				//For now, only hungarian points
				$result = array(
					'id' => $place['tof_shop_id'],
					'lat' => number_format(str_replace(',', '.', $place['gis_y']), 5, '.', ''),
					'lon' => number_format(str_replace(',', '.', $place['gis_x']), 5, '.', ''),
					'name' => $place['name'],
					'zip' => $place['zip_code'],
					'addr' => $place['street'],
					'city' => $place['city']
				);

				$results[] = $result;
			}

			//Save stuff
			self::save_json_file('expressone', $results);
			return true;

		}

		public static function get_gls_json() {

			//Make request for json file
			$request = wp_remote_get('https://csomag.hu/api/parcel-shops');

			//Check for errors
			if( is_wp_error( $request ) ) {
				VP_Woo_Pont()->log_error_messages($request, 'gls-import-points');
				return false;
			}

			//Get body
			$body = wp_remote_retrieve_body( $request );

			//Try to convert into json
			$json = json_decode( $body, true );

			//Check if json exists
			if($json === null) {
				return false;
			}

			//Create a new json
			$results = array();

			//Simplify json, so its smaller to store, faster to load
			foreach ($json['data'] as $place) {

				//For now, only hungarian points
				$result = array(
					'id' => $place['id'],
					'lat' => number_format($place['location'][0], 5, '.', ''),
					'lon' => number_format($place['location'][1], 5, '.', ''),
					'name' => $place['name'],
					'zip' => $place['contact']['zipCode'],
					'addr' => $place['contact']['address'],
					'city' => $place['contact']['city']
				);

				if($place['description']) {
					$result['comment'] = $place['description'];
				}

				$results[] = $result;
			}

			//Save stuff
			self::save_json_file('gls', $results);
			return true;

		}

		public static function save_json_file($provider, $json) {

			//Create smaller json
			$smaller_json = json_encode($json, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);

			//Get filename and download folder
			$paths = VP_Woo_Pont_Helpers::get_download_folder($provider);
			$filename = $paths['name'];

			//Remove existing file
			$existing_filename = get_option('_vp_woo_pont_file_'.$provider);
			if($existing_filename) {
				$existing_file = $paths['dir'].$existing_filename;
				if(file_exists($existing_file)){
					unlink($existing_file);
				}
			}

			//Save the file
			file_put_contents($paths['path'], $smaller_json);

			//Store current file name in db
			update_option('_vp_woo_pont_file_'.$provider, $filename);

		}

		public static function import_manually() {
			check_ajax_referer( 'vp-woo-pont-import-json-manually', 'nonce' );

			//Get provider id
			$pont_type = sanitize_text_field($_POST['provider']);

			//Try to run the import
			$import = call_user_func(array(__CLASS__, 'get_'.$pont_type.'_json'));

			//Return response
			if($import) {
				$download_folders = VP_Woo_Pont_Helpers::get_download_folder();
				$updated_file = get_option('_vp_woo_pont_file_'.$pont_type);
				wp_send_json_success(array(
					'url' => $download_folders['url'].$updated_file,
					'message' => __('Import run successfully.', 'vp-woo-pont')
				));
			} else {
				wp_send_json_error(array(
					'message' => __('Unable to run the import tool for some reason.', 'vp-woo-pont')
				));
			}

		}

	}

 VP_Woo_Pont_Import_Database::init();

endif;
