<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VP_Woo_Pont_Foxpost {
	protected $api_url = 'https://api.csomagvarazslo.hu/v2/';
	protected $username = '';
	protected $api_key = '';
	protected $webhook_url = '';

	public function __construct() {
		$this->username = VP_Woo_Pont_Helpers::get_option('foxpost_username');
		$this->api_key = VP_Woo_Pont_Helpers::get_option('foxpost_api_key');
		$this->webhook_url = get_option( '_vp_woo_pont_foxpost_webhook_url');

		//Process IPN requests
		add_action( 'admin_post_vp_woo_pont_webhook_foxpost', array( $this, 'ipn_process' ) );
		add_action( 'admin_post_nopriv_vp_woo_pont_webhook_foxpost', array( $this, 'ipn_process' ) );

	}

	public function create_label($data) {

		//Create item
		$item = array(
			'apiExternalId' => $data['order_id'],
			'recipientName' => $data['customer']['name'],
			'recipientPhone' => $data['customer']['phone'],
			'recipientEmail' => $data['customer']['email'],
			'aptIdentifier' => $data['point_id'],
			'parcelSizeId' => VP_Woo_Pont_Helpers::get_option('foxpost_parcel_size', 'XS'),
			'parcelType' => 'apt'
		);

		//Check for COD
		if($data['package']['cod']) {
			$item['cashOnDeliveryMoney'] = $data['package']['total'];
		}

		//Create request data
		$options = array(
			'items' => array($item)
		);

		//So developers can modify
		$options = apply_filters('vp_woo_pont_foxpost_label', $options, $data);
		$auth_header = apply_filters('vp_woo_pont_foxpost_auth_header', 'Basic ' . base64_encode( $this->username . ':' . $this->api_key ), $options);

		//Logging
		VP_Woo_Pont()->log_debug_messages($options, 'foxpost-create-label');

		//Submit request
		$request = wp_remote_post( $this->api_url.'parcel/mass-create-parcel', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Authorization' => $auth_header,
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Check for API errors
		if(wp_remote_retrieve_response_code( $request ) != 202) {
			VP_Woo_Pont()->log_error_messages($response, 'foxpost-create-label');
			$response = wp_remote_retrieve_body( $request );
			$response = json_decode( $response, true );
			if($response && $response['errorMessage']) {
				return new WP_Error( 'foxpost_error_'.$response['error'], $response['errorMessage'] );
			} else {
				return new WP_Error( 'foxpost_error_unknown', __('Unknown error', 'vp-woo-pont') );
			}
		}

		//Check for success
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );
		$label = array();
		$label['success'] = true;
		$label['pending'] = true;

		return $label;
	}

	public function void_label($data) {

		//Create request data
		$options = array(
			'items' => array(
				array(
					'clfoxId' => $data['parcel_number'],
					'apiExternalId' => $data['order_id']
				)
			)
		);

		VP_Woo_Pont()->log_debug_messages($options, 'foxpost-void-label-request');

		//So developers can modify
		$options = apply_filters('vp_woo_pont_foxpost_void_label', $options, $data);
		$auth_header = apply_filters('vp_woo_pont_foxpost_auth_header', 'Basic ' . base64_encode( $this->username . ':' . $this->api_key ), $options);

		//Submit request
		$request = wp_remote_post( $this->api_url.'parcel/mass-delete-parcel', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Authorization' => $auth_header,
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Check for API errors
		if(wp_remote_retrieve_response_code( $request ) != 202) {
			VP_Woo_Pont()->log_error_messages($response, 'foxpost-delete-label');
			$response = wp_remote_retrieve_body( $request );
			$response = json_decode( $response, true );
			if($response && $response['errorMessage']) {
				return new WP_Error( 'foxpost_error_'.$response['error'], $response['errorMessage'] );
			} else {
				return new WP_Error( 'foxpost_error_unknown', __('Unknown error', 'vp-woo-pont') );
			}
		}

		//Check for success
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );
		$label = array();
		$label['success'] = true;

		VP_Woo_Pont()->log_debug_messages($response, 'foxpost-void-label-response');

		return $label;
	}

	//Process webhook request
	public function ipn_process() {

		//Set response header
		status_header(200);

		//Retrieve JSON payload
		$data = json_decode(file_get_contents('php://input'));

		//Check if data exists
		if(!$data) {
			exit();
		}

		//Logging
		VP_Woo_Pont()->log_debug_messages($data, 'foxpost-ipn-process');

		//Check for webhook action, if not create-parcel, cancel processing
		if(!isset($data->webHookAction) && !$data->webHookAction == 'create-parcel') {
			exit();
		}

		//API returns an array, we only need one item
		$item = $data->items[0];

		//Get order
		$order_id = $item->apiExternalId;
		$order = wc_get_order($order_id);

		//Make sure the order still exists
		if(!$order) {
			exit();
		}

		//Check for errors. If there is an error, log it
		if($item->error) {
			$error_code = $item->message;
			$order->add_order_note(sprintf(esc_html__('Unable to create shipping label. Error code: %s', 'vp-woo-pont'), urldecode($error_code)));
			$order->delete_meta_data('_vp_woo_pont_parcel_id');
			$order->delete_meta_data('_vp_woo_pont_parcel_pdf');
			$order->delete_meta_data('_vp_woo_pont_parcel_number');
			$order->delete_meta_data('_vp_woo_pont_parcel_pending');
			$order->save();
			exit();
		}

		//If all goes well, get the package info, ids and stuff
		$parcel_id = $item->parcelSendingCode;
		$parcel_number = $item->clfoxId;

		//Save these infos
		$order->update_meta_data('_vp_woo_pont_parcel_id', $parcel_id);
		$order->update_meta_data('_vp_woo_pont_parcel_number', $parcel_number);
		$order->save();

		//And try to generate a label
		$options = array(
			'stickerSize' => VP_Woo_Pont_Helpers::get_option('foxpost_sticker_size', 'a7'),
			'items' => array(
				array(
					'clfoxId' => $parcel_number
				)
			)
		);

		//Submit request
		$request = wp_remote_post( $this->api_url.'parcel/create-sticker', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Authorization' => 'Basic ' . base64_encode( $this->username . ':' . $this->api_key ),
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
		));

		//Check for errors
		$has_error = false;
		if(is_wp_error($request)) {
			$has_error = $request;
		}

		//Get response
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );

		//Check for API errors
		if(wp_remote_retrieve_response_code( $request ) != 200) {
			VP_Woo_Pont()->log_error_messages($response, 'foxpost-download-label');
			if($response && $response['error']) {
				$has_error = new WP_Error( 'foxpost_create_sticker_error', $response['message'] );
			} else {
				$has_error = new WP_Error( 'foxpost_create_sticker_error', __('Unknown error', 'vp-woo-pont') );
			}
		}

		//Check for erros
		if($has_error) {
			$order->add_order_note(sprintf(esc_html__('Unable to create shipping label. Error code: %s', 'vp-woo-pont'), urldecode($has_error->get_error_message())));
			exit();
		}

		//Get PDF file content
		$attachment = $response['attachments'][0];
		$pdf = base64_decode($attachment['content']);

		//Try to save PDF file
		$pdf_file = VP_Woo_Pont_Labels::get_pdf_file_path('foxpost', $order_id);
		VP_Woo_Pont_Labels::save_pdf_file($pdf, $pdf_file);

		//Update meta
		$order->update_meta_data('_vp_woo_pont_parcel_pending', false);
		$order->update_meta_data('_vp_woo_pont_parcel_pdf', $pdf_file['name']);
		$order->save();
		$order->add_order_note(sprintf(esc_html__('Shipping label generated successfully. ID number: %s', 'vp-woo-pont'), $parcel_number));

		//So developers can hook in here
		do_action('vp_woo_pont_label_created', $order, array('number' => $parcel_number), 'foxpost');

		exit();
	}

	//Return tracking link
	public function get_tracking_link($parcel_number) {
		return 'https://foxpost.hu/csomagkovetes/?code='.esc_attr($parcel_number);
	}

}
